const {
  formatPerfLine,
  formatHeader,
  formatDividerLine,
} = require("./format-helpers");

function getHeader(pLevel, pMaxLevel) {
  if (pLevel <= pMaxLevel) {
    return formatHeader();
  }
  return "";
}

function getProgressLine(pMessage, pState, pLevel, pMaxLevel) {
  let lReturnValue = "";

  if (pLevel <= pMaxLevel) {
    const lTime = process.uptime();
    const { user, system } = process.cpuUsage();
    const { rss, heapTotal, heapUsed, external } = process.memoryUsage();
    const lStats = {
      elapsedTime: lTime - pState.previousTime,
      elapsedUser: user - pState.previousUserUsage,
      elapsedSystem: system - pState.previousSystemUsage,
      deltaRss: rss - pState.previousRss,
      deltaHeapUsed: heapUsed - pState.previousHeapUsed,
      deltaHeapTotal: heapTotal - pState.previousHeapTotal,
      deltaExternal: external - pState.previousExternal,
      message: pState.previousMessage,
    };

    pState.previousMessage = pMessage;
    pState.previousTime = lTime;
    pState.previousUserUsage = user;
    pState.previousSystemUsage = system;
    pState.previousRss = rss;
    pState.previousHeapTotal = heapTotal;
    pState.previousHeapUsed = heapUsed;
    pState.previousExternal = external;

    lReturnValue = formatPerfLine(lStats);
  }
  return lReturnValue;
}

function getEndText(pState, pLevel, pMaxLevel) {
  if (pLevel <= pMaxLevel) {
    const lTime = process.uptime();
    const { user, system } = process.cpuUsage();
    const { heapUsed, heapTotal, external, rss } = process.memoryUsage();
    pState.previousMessage = "really done";

    return (
      getProgressLine("", pState, pLevel, pMaxLevel) +
      formatDividerLine() +
      formatPerfLine({
        elapsedTime: lTime,
        elapsedUser: user,
        elapsedSystem: system,
        deltaRss: rss,
        deltaHeapUsed: heapUsed,
        deltaHeapTotal: heapTotal,
        deltaExternal: external,
        message: "",
      })
    );
  }
  return "";
}

module.exports = {
  getHeader,
  getProgressLine,
  getEndText,
};
